@@ -67,6 +67,7 @@ int main(int argc, char **argv) {
 	char file_version[3]={DACT_VER_MAJOR, DACT_VER_MINOR, DACT_VER_REVISION};
 	char option_verbose=0;
 	char option_complain=0;
+	char option_old=0;
 	char ch;
 	FOUR_BYTES file_size_real=0, file_size, out_file_size, in_magic;
 	int stat_cnt[256];
@@ -107,7 +108,7 @@ int main(int argc, char **argv) {
 				mode=DACT_MODE_DECMP;
 				break;
 			case 'f':
-				mode_force=1;
+				mode_force++;
 				break;
 			case 'v':
 				option_verbose++;
@@ -177,6 +178,10 @@ int main(int argc, char **argv) {
 			strncpy(out_filename,argv[1+optind],sizeof(out_filename)-1);
 		} else {
 			if (isatty(STDOUT_FILENO)) {
+				if (isatty(in_fd)) {
+					fprintf(stderr, "%s: In and out are both terminals.\n",argv[0]);
+					return(-1);
+				}
 				if (mode==DACT_MODE_COMPR) {
 					strncpy(out_filename,in_filename,sizeof(out_filename)-4);
 					strcat(out_filename,".dct");
@@ -285,19 +290,34 @@ int main(int argc, char **argv) {
 		case DACT_MODE_DECMP:
 			i=x=0;
 			x+=read(in_fd,&file_version,sizeof(file_version));
-			x+=READ_4BYTE_DE(in_fd,in_magic);
-			if (in_magic!=DACT_MAGIC_NUMBER) {
+
+			if ( ((file_version[0]<<16)|(file_version[1]<<8)|file_version[2])  <= 0x000402) {
+				fprintf(stderr, "Warning: Old Format\n");
+				
+				option_old=1;
+				if (file_version[2]==2) option_old=2;
+				in_magic=DACT_MAGIC_NUMBER;
+			} else {
+				x+=READ_4BYTE_DE(in_fd,in_magic);
+			}
+
+			if (in_magic!=DACT_MAGIC_NUMBER && mode_force<2) {
 				printf("%s: Bad magic.\n", argv[0]);
 				return(-1);
 			}
-			x+=READ_4BYTE_DE(in_fd,out_file_size);
-			x+=READ_2BYTE_DE(in_fd,DACT_BLK_SIZE);
-//			x+=read(in_fd,&out_file_size,sizeof(out_file_size));
-//			x+=read(in_fd,&DACT_BLK_SIZE,sizeof(DACT_BLK_SIZE));
+
+			if (option_old) {
+				x+=read(in_fd,&out_file_size,sizeof(out_file_size));
+				if (option_old==2) { x+=read(in_fd,&DACT_BLK_SIZE,sizeof(int)); } else { DACT_BLK_SIZE=1024; }
+			} else {
+				x+=READ_4BYTE_DE(in_fd,out_file_size);
+				x+=READ_2BYTE_DE(in_fd,DACT_BLK_SIZE);
+			}
+
 			x+=read(in_fd,&cmp_options,sizeof(cmp_options));
 //			x=sizeof(out_file_size)+sizeof(cmp_options);
 
-			if (file_version[0]!=DACT_VER_MAJOR) {
+			if (file_version[0]!=DACT_VER_MAJOR && mode_force<2) {
 				fprintf(stderr, "Major version numbers do not match! (%i!=%i)\n",file_version[0],DACT_VER_MAJOR);
 				fprintf(stderr, "File is not usable, exiting...\n");
 				close(in_fd);
@@ -312,8 +332,11 @@ int main(int argc, char **argv) {
 
 			while(1) {
 				i++;
-//				read(in_fd,&header.size,sizeof(header.size));
-				i+=2-READ_2BYTE_DE(in_fd,header.size);
+				if (option_old) {
+					read(in_fd,&header.size,sizeof(header.size));
+				} else {
+					i+=2-READ_2BYTE_DE(in_fd,header.size);
+				}
 				read(in_fd,&header.algo,sizeof(header.algo));
 				read(in_fd,&in_buffer,header.size);
 				x+=sizeof(header.size)+sizeof(header.algo)+header.size;
@@ -347,26 +370,54 @@ int main(int argc, char **argv) {
 			break;
 		case DACT_MODE_STAT:
 			x=read(in_fd,&file_version,sizeof(file_version));
-			x+=READ_4BYTE_DE(in_fd,in_magic);
-			x+=READ_4BYTE_DE(in_fd,out_file_size);
-			x+=READ_2BYTE_DE(in_fd,DACT_BLK_SIZE);
+
+			if ( ((file_version[0]<<16)|(file_version[1]<<8)|file_version[2])  <= 0x000402) {
+				fprintf(stderr, "Warning: Old file format.\n");
+				option_old=1;
+				x+=read(in_fd,&out_file_size,sizeof(out_file_size));
+				if (file_version[2]==2) {
+					x+=read(in_fd,&DACT_BLK_SIZE,sizeof(int));
+				} else {
+					DACT_BLK_SIZE=1024;
+				}
+
+				in_magic=DACT_MAGIC_NUMBER;
+			} else {
+				x+=READ_4BYTE_DE(in_fd,in_magic);
+				x+=READ_4BYTE_DE(in_fd,out_file_size);
+				x+=READ_2BYTE_DE(in_fd,DACT_BLK_SIZE);
+			}
 //			x+=read(in_fd,&out_file_size,sizeof(out_file_size));
 //			x+=read(in_fd,&DACT_BLK_SIZE,sizeof(DACT_BLK_SIZE));
 			x+=read(in_fd,&cmp_options,sizeof(cmp_options));
 
-			printf("%s: File              = %s\n",argv[0],in_filename);
-			printf("%s: Magic             = %u\n",argv[0],in_magic);
-			printf("%s: DACT Version      = %i.%i.%i\n",argv[0],file_version[0],file_version[1],file_version[2]);
-			printf("%s: DACT Block Size   = %i\n",argv[0],DACT_BLK_SIZE);
-			printf("%s: Compressed Size   = %u\n",argv[0],file_size);
-			printf("%s: Uncompressed Size = %u\n",argv[0],out_file_size);
-			printf("%s: Ratio             = %f %%\n",argv[0],100.0-((((float) (file_size))/((float) (out_file_size)))*100));
+			printf("File              = %s\n",in_filename);
+			printf("Magic             = 0x%08x",in_magic);
+			if (in_magic!=DACT_MAGIC_NUMBER && mode_force<2) {
+				printf(" (Bad magic.)\n");
+				close(in_fd);
+				close(out_fd);
+				return(-1);
+			} else {
+				printf("\n");
+			}
+
+			printf("DACT version      = %i.%i.%i\n",file_version[0],file_version[1],file_version[2]);
+			printf("Block size        = %i\n",DACT_BLK_SIZE);
+			printf("Compressed size   = %u\n",file_size);
+			printf("Uncompressed size = %u\n",out_file_size);
+			printf("Ratio             = %2.3f%%\n",100.0-((((float) (file_size))/((float) (out_file_size)))*100));
+
+
 			if (option_verbose!=0) {
-				printf("%s: NUMBER | COUNT | NAME\n",argv[0]);
-				printf("%s: -------+-------+----------------------------------------\n",argv[0]);
+				printf("NUMBER | COUNT | NAME\n");
+				printf("-------+-------+----------------------------------------\n");
 				while (1) {
-//					x+=read(in_fd,&header.size,sizeof(header.size));
-					x+=READ_2BYTE_DE(in_fd,header.size);
+					if (option_old) {
+						x+=read(in_fd,&header.size,sizeof(header.size));
+					} else {
+						x+=READ_2BYTE_DE(in_fd,header.size);
+					}
 					x+=read(in_fd,&header.algo,sizeof(header.algo));
 					x+=header.size;
 					lseek(in_fd,header.size,SEEK_CUR);
@@ -375,7 +426,7 @@ int main(int argc, char **argv) {
 				}
 				for (i=0;i<256;i++) {
 					if (algorithms[i]==NULL) break;
-					printf("%s: %6i | %5i | %s\n", argv[0],i,stat_cnt[i],algorithm_names[i]);
+					printf("%6i | %5i | %s\n",i,stat_cnt[i],algorithm_names[i]);
 				}
 			}
 			break;			
